@@ -25,6 +25,7 @@
 #include <QCoreApplication>
 #include <QDate>
 #include <QDebug>
+#include <QMessageBox>
 #include <QSettings>
 
 
@@ -41,6 +42,7 @@ Settings::Settings(const QCommandLineParser &arg_parser)
     processArgs(arg_parser);
     if (arg_parser.isSet("month"))
         setMonthlySnapshot(arg_parser);
+    override_size = arg_parser.isSet("override-size");
     processExclArgs(arg_parser);
 }
 
@@ -51,55 +53,51 @@ Settings::~Settings()
 // check if compression is available in the kernel (lz4, lzo, xz)
 bool Settings::checkCompression()
 {
-    if (QFileInfo::exists("/boot/config-" + kernel)) { // return true if cannot check config file
+    if (!QFileInfo::exists("/boot/config-" + kernel)) // return true if cannot check config file
         return true;
-    }
-    if (compression == "lz4") {
-        return (shell->run("grep ^CONFIG_SQUASHFS_LZ4=y /boot/config-" + kernel.toUtf8()));
-    } else if (compression == "xz") {
-        return (shell->run("grep ^CONFIG_SQUASHFS_XZ=y /boot/config-" + kernel.toUtf8()));
-    } else if (compression == "lzo") {
-        return (shell->run("grep ^CONFIG_SQUASHFS_LZO=y /boot/config-" + kernel.toUtf8()));
-    } else {
-        return true;
-    }
+
+    if (compression == "lz4")
+        return (shell->run("grep ^CONFIG_SQUASHFS_LZ4=y /boot/config-" + kernel));
+    else if (compression == "xz")
+        return (shell->run("grep ^CONFIG_SQUASHFS_XZ=y /boot/config-" + kernel));
+    else if (compression == "lzo")
+        return (shell->run("grep ^CONFIG_SQUASHFS_LZO=y /boot/config-" + kernel));
+    else if (compression == "zstd")
+        return (shell->run("grep ^CONFIG_SQUASHFS_ZSTD=y /boot/config-" + kernel));
+    return true;
 }
 
 // adds or removes exclusion to the exclusion string
 void Settings::addRemoveExclusion(bool add, QString exclusion)
 {
-    if (exclusion.startsWith("/")) {
+    if (exclusion.startsWith("/"))
         exclusion.remove(0, 1); // remove preceding slash
-    }
     if (add) {
-        if (session_excludes.isEmpty()) {
+        if (session_excludes.isEmpty())
             session_excludes.append("-e \"" + exclusion + "\"");
-        } else {
+        else
             session_excludes.append(" \"" + exclusion + "\"");
-        }
     } else {
         session_excludes.remove(" \"" + exclusion + "\"");
-        if (session_excludes == "-e") {
+        if (session_excludes == "-e")
             session_excludes = "";
-        }
     }
 }
 
 bool Settings::checkSnapshotDir()
 {
     qDebug() << "+++" << __PRETTY_FUNCTION__ << "+++";
-    if (!snapshot_dir.mkpath(snapshot_dir.absolutePath())) {
-        qDebug() << QObject::tr("Could not create working directory. ") + snapshot_dir.absolutePath();
+    if (!QDir().mkpath(snapshot_dir)) {
+        qDebug() << QObject::tr("Could not create working directory. ") + snapshot_dir;
         return false;
     }
-    system("chown $(logname):$(logname) \"/" + snapshot_dir.absolutePath().toUtf8() + "\"");
+    system("chown $(logname):$(logname) \"/" + snapshot_dir.toUtf8() + "\"");
     // Create a work_dir
-    tempdir_parent = snapshot_dir.absolutePath();
-    if (!isOnSupportedPart(snapshot_dir)) { // if not saving snapshot on a Linux partition put working dir in /tmp or /home
+    tempdir_parent = snapshot_dir;
+    if (!isOnSupportedPart(snapshot_dir)) // if not saving snapshot on a Linux partition put working dir in /tmp or /home
         tempdir_parent = largerFreeSpace("/tmp", "/home");
-    } else {
-        tempdir_parent = largerFreeSpace("/tmp", "/home", snapshot_dir.absolutePath());
-    }
+    else
+        tempdir_parent = largerFreeSpace("/tmp", "/home", snapshot_dir);
     return true;
 }
 
@@ -107,7 +105,7 @@ bool Settings::checkTempDir()
 {
     qDebug() << "+++" << __PRETTY_FUNCTION__ << "+++";
     tmpdir.reset(new QTemporaryDir(tempdir_parent + "/iso-snapshot-XXXXXXXX"));
-    if(!tmpdir->isValid()) {
+    if (!tmpdir->isValid()) {
         qDebug() << QObject::tr("Could not create temp directory. ") + tmpdir.data()->path();
         return false;
     }
@@ -127,12 +125,13 @@ QString Settings::getEditor()
 {
     QString editor;
     if (system("command -v " + gui_editor.fileName().toUtf8()) != 0) {  // if specified editor doesn't exist get the default one
-        editor = shell->getCmdOut("grep Exec $(locate $(xdg-mime query default text/plain)) |cut -d= -f2 |cut -d' ' -f1 |head -1");
-        if (editor == "kate" || editor == "kwrite") { // need to run these as normal user
-            editor = "runuser -u $(logname) " + editor;
-        }
+        QString local = QFile::exists(QDir::homePath() + "/.local/share/applications") ? "/.local/share/applications " : " ";
+        QString desktop_file = shell->getCmdOut("find " + local + "/usr/share/applications -name $(xdg-mime query default text/plain) |grep -m1 .");
+        editor = shell->getCmdOut("grep -m1 ^Exec " + desktop_file + " |cut -d= -f2 |cut -d\" \" -f1", true);
         if (editor.isEmpty() || system("command -v " + editor.toUtf8()) != 0) { // if default one doesn't exit use nano as backup editor
             editor = "x-terminal-emulator -e nano";
+        } else if (editor == "kate" || editor == "kwrite") { // need to run these as normal user
+            editor = "runuser -u $(logname) " + editor;
         }
     } else {
         editor = gui_editor.fileName();
@@ -144,12 +143,11 @@ QString Settings::getEditor()
 QString Settings::getSnapshotSize()
 {
     QString size;
-    if (snapshot_dir.exists()) {
-        QString cmd = QString("find \"%1\" -maxdepth 1 -type f -name '*.iso' -exec du -shc {} + |tail -1 |awk '{print $1}'").arg(snapshot_dir.absolutePath());
+    if (QFileInfo::exists(snapshot_dir)) {
+        QString cmd = QString("find \"%1\" -maxdepth 1 -type f -name '*.iso' -exec du -shc {} + |tail -1 |awk '{print $1}'").arg(snapshot_dir);
         size = shell->getCmdOut(cmd);
-        if (!size.isEmpty()) {
+        if (!size.isEmpty())
             return size;
-        }
     }
     return "0";
 }
@@ -157,8 +155,8 @@ QString Settings::getSnapshotSize()
 // return number of snapshots in snapshot_dir
 int Settings::getSnapshotCount()
 {
-    if (snapshot_dir.exists()) {
-        QFileInfoList list = snapshot_dir.entryInfoList(QStringList("*.iso"), QDir::Files);
+    if (QFileInfo::exists(snapshot_dir)) {
+        QFileInfoList list = QDir(snapshot_dir).entryInfoList(QStringList("*.iso"), QDir::Files);
         return list.size();
     }
     return 0;
@@ -167,8 +165,8 @@ int Settings::getSnapshotCount()
 quint64 Settings::getFreeSpace(const QString &path)
 {
     bool ok;
-    quint64 result = shell->getCmdOut(QString("df -k --output=avail \"%1\" |tail -n1").arg(path)).toUInt(&ok);
-    if (not ok) {
+    quint64 result = shell->getCmdOut(QString("df -k --output=avail \"%1\" |tail -n1").arg(path)).toULongLong(&ok);
+    if (!ok) {
         qDebug() << "Can't calculate free space on" << path;
         return 0;
     }
@@ -181,18 +179,20 @@ QString Settings::getXdgUserDirs(const QString& folder)
     QString result = "";
 
     for (const QString &user : users) {
-        QString out;
-        bool success = shell->run("runuser -l " + user + " -c \"xdg-user-dir " + folder + "\"", out);
-        QString dir = QString(out);
+        QString dir;
+        bool success = shell->run("runuser -l " + user + " -c \"xdg-user-dir " + folder + "\"", dir);
         if (success) {
-            if (englishDirs.value(folder) == dir.section("/", -1) || dir.trimmed() == "/home/" + user || dir.trimmed() == "/home/" + user + "/" || dir.isEmpty()) { // skip if English name or of return folder is the home folder (if XDG-USER-DIR not defined)
+            if (englishDirs.value(folder) == dir.section("/", -1)
+                    || dir.trimmed() == "/home/" + user
+                    || dir.trimmed() == "/home/" + user + "/"
+                    || dir.isEmpty()) // skip if English name or of return folder is the home folder (if XDG-USER-DIR not defined)
                 continue;
-            }
-            if (dir.startsWith("/")) {
+            if (dir.startsWith("/"))
                 dir.remove(0, 1); // remove training slash
-            }
-            (folder == "DESKTOP") ? dir.append("/!(minstall.desktop)") : dir.append("/*\" \"" + dir + "/.*");
-            (result.isEmpty()) ? result.append("\" \"" + dir) : result.append(" \"" + dir);
+            (folder == "DESKTOP") ? dir.append("/!(minstall.desktop)")
+                                  : dir.append("/*\" \"" + dir + "/.*");
+            (result.isEmpty()) ? result.append("\" \"" + dir)
+                               : result.append(" \"" + dir);
             result.append("\""); // close the quote for each user, will strip the last one before returning;
         }
     }
@@ -206,9 +206,9 @@ void Settings::selectKernel()
     if (kernel.isEmpty() || !QFileInfo::exists("/boot/vmlinuz-" + kernel)) {  // if kernel version not passed as arg, or incorrect
         kernel = shell->getCmdOut("uname -r");
         if (!QFileInfo::exists("/boot/vmlinuz-" + kernel)) { // if current kernel doesn't exist for some reason (e.g. WSL) in /boot pick first kernel
-             kernel = shell->getCmdOut("ls -1 /boot/vmlinuz* | sort | tail -n1").remove("/boot/vmlinuz-");
+             kernel = shell->getCmdOut("ls -1 /boot/vmlinuz* |sort |tail -n1").remove("/boot/vmlinuz-");
              if (!QFileInfo::exists("/boot/vmlinuz-" + kernel)) {
-                 qDebug() << QObject::tr("Could not find a usable kernel");
+                 QMessageBox::critical(nullptr, QObject::tr("Error"), QObject::tr("Could not find a usable kernel"));
                  exit(EXIT_FAILURE);
              }
         }
@@ -249,7 +249,7 @@ QString Settings::getFilename()
         int n = 1;
         do {
             name = snapshot_basename + QString::number(n) + ".iso";
-            dir.setPath("\"" + snapshot_dir.absolutePath() + "/" + name + "\"");
+            dir.setPath("\"" + snapshot_dir + "/" + name + "\"");
             n++;
         } while (QFileInfo::exists(dir.absolutePath()));
         return name;
@@ -267,7 +267,8 @@ quint64 Settings::getLiveRootSpace()
     QString sqfile_full = livesettings.value("SQFILE_FULL", "/live/boot-dev/antiX/linuxfs").toString();
 
     // get compression factor by reading the linuxfs squasfs file, if available
-    QString linuxfs_compression_type = shell->getCmdOut("dd if=" + sqfile_full + " bs=1 skip=20 count=2 status=none 2>/dev/null |od -An -tdI");
+    QString linuxfs_compression_type = shell->getCmdOut("dd if=" + sqfile_full
+                                            + " bs=1 skip=20 count=2 status=none 2>/dev/null |od -An -tdI").trimmed();
     uint c_factor;
     // gzip, xz, or lz4
     if (linuxfs_compression_type == "1") c_factor = compression_factor.value("gzip");
@@ -275,13 +276,13 @@ quint64 Settings::getLiveRootSpace()
     else if (linuxfs_compression_type == "3") c_factor = compression_factor.value("lzma"); // lzma, not used by antiX
     else if (linuxfs_compression_type == "4") c_factor = compression_factor.value("xz");
     else if (linuxfs_compression_type == "5") c_factor = compression_factor.value("lz4");
+    else if (linuxfs_compression_type == "6") c_factor = compression_factor.value("zstd");
     else c_factor = 30; //anything else or linuxfs not reachable (toram), should be pretty conservative
 
     quint64 rootfs_file_size = 0;
-    quint64 linuxfs_file_size = shell->getCmdOut("df -k /live/linux --output=used --total |tail -n1").toUInt() * 100 / c_factor;
-    if (QFileInfo::exists("/live/persist-root")) {
-        rootfs_file_size = shell->getCmdOut("df -k /live/persist-root --output=used --total |tail -n1").toUInt();
-    }
+    quint64 linuxfs_file_size = shell->getCmdOut("df -k /live/linux --output=used --total |tail -n1").toULongLong() * 100 / c_factor;
+    if (QFileInfo::exists("/live/persist-root"))
+        rootfs_file_size = shell->getCmdOut("df -k /live/persist-root --output=used --total |tail -n1").toULongLong();
 
     // add rootfs file size to the calculated linuxfs file size.  probaby conservative, as rootfs will likely have some overlap with linuxfs
     return linuxfs_file_size + rootfs_file_size;
@@ -295,13 +296,13 @@ QString Settings::getUsedSpace()
         root_size = getLiveRootSpace();
         out += QString::number(root_size / 1048576.0, 'f', 2) + "GiB" + " -- " + QObject::tr("estimated");
     } else {
-        root_size = shell->getCmdOut("df -k --output=used / |tail -n1").toUInt(&ok);
-        if (not ok) return "Can't calculate free space on root";
+        root_size = shell->getCmdOut("df -k --output=used / |tail -n1").toULongLong(&ok);
+        if (!ok) return "Can't calculate free space on root";
         out += QString::number(root_size / 1048576.0, 'f', 2) + "GiB";
     }
     if (shell->run("mountpoint -q /home")) {
-        home_size = shell->getCmdOut("df -k --output=used /home |tail -n1").toUInt(&ok);
-        if (not ok) return "Can't calculate free space on /home";
+        home_size = shell->getCmdOut("df -k --output=used /home |tail -n1").toULongLong(&ok);
+        if (!ok) return "Can't calculate free space on /home";
         out.append("\n- " + QObject::tr("Used space on /home: ") + QString::number(home_size / 1048576.0, 'f', 2) + "GiB");
     } else {
         home_size = 0; // /home on root
@@ -323,11 +324,12 @@ bool Settings::isLive()
 }
 
 // checks if the directory is on a Linux partition
-bool Settings::isOnSupportedPart(const QDir &dir)
+bool Settings::isOnSupportedPart(const QString &dir)
 {
     qDebug() << "+++" << __PRETTY_FUNCTION__ << "+++";
-    QStringList supported_partitions = (QStringList() << "ext2/ext3" << "btrfs" << "jfs" << "reiserfs" << "xfs" << "fuseblk"); // supported partition types (NTFS returns fuseblk)
-    QString part_type = shell->getCmdOut("stat --file-system --format=%T \"" + dir.absolutePath() + "\"").trimmed();
+    // Supported partition types (NTFS returns fuseblk)
+    QStringList supported_partitions = (QStringList() << "ext2/ext3" << "btrfs" << "jfs" << "reiserfs" << "xfs" << "fuseblk");
+    QString part_type = shell->getCmdOut("stat --file-system --format=%T \"" + dir + "\"").trimmed();
     qDebug() << "detected partition" << part_type << "supported part:" << supported_partitions.contains(part_type);
     return supported_partitions.contains(part_type);
 }
@@ -336,7 +338,8 @@ bool Settings::isOnSupportedPart(const QDir &dir)
 QString Settings::largerFreeSpace(const QString &dir1, const QString &dir2)
 {
     qDebug() << "+++" << __PRETTY_FUNCTION__ << "+++";
-    if (shell->getCmdOut("stat -c '%d' " + dir1) == shell->getCmdOut("stat -c '%d' " + dir2)) return dir1;
+    if (shell->getCmdOut("stat -c '%d' " + dir1) == shell->getCmdOut("stat -c '%d' " + dir2))
+        return dir1;
     quint64 dir1_free = getFreeSpace(dir1);
     quint64 dir2_free = getFreeSpace(dir2);
     return dir1_free >= dir2_free ? dir1 : dir2;
@@ -371,13 +374,13 @@ QStringList Settings::listUsers()
 
 void Settings::excludeItem(const QString &item)
 {
-    if (item == QObject::tr("Desktop") or item == "Desktop") excludeDesktop(true);
-    if (item == QObject::tr("Documents") or item == "Documents") excludeDocuments(true);
-    if (item == QObject::tr("Downloads") or item == "Downloads") excludeDownloads(true);
-    if (item == QObject::tr("Music") or item == "Music") excludeMusic(true);
-    if (item == QObject::tr("Networks") or item == "Networks") excludeNetworks(true);
-    if (item == QObject::tr("Pictures") or item == "Pictures") excludePictures(true);
-    if (item == QObject::tr("Videos") or item == "Videos") excludeVideos(true);
+    if (item == QObject::tr("Desktop") || item == "Desktop") excludeDesktop(true);
+    if (item == QObject::tr("Documents") || item == "Documents") excludeDocuments(true);
+    if (item == QObject::tr("Downloads") || item == "Downloads") excludeDownloads(true);
+    if (item == QObject::tr("Music") || item == "Music") excludeMusic(true);
+    if (item == QObject::tr("Networks") || item == "Networks") excludeNetworks(true);
+    if (item == QObject::tr("Pictures") || item == "Pictures") excludePictures(true);
+    if (item == QObject::tr("Videos") || item == "Videos") excludeVideos(true);
 }
 
 void Settings::excludeDesktop(bool exclude)
@@ -458,6 +461,8 @@ void Settings::loadConfig()
 
     session_excludes = "";
     snapshot_dir = settings.value("snapshot_dir", "/home/snapshot").toString();
+    if (!snapshot_dir.endsWith("/snapshot"))
+        snapshot_dir += (snapshot_dir.endsWith("/") ? "snapshot" : "/snapshot");
     snapshot_excludes.setFileName(settings.value("snapshot_excludes", "/usr/local/share/excludes/iso-snapshot-exclude.list").toString());
     snapshot_basename = settings.value("snapshot_basename", "snapshot").toString();
     make_chksum = settings.value("make_md5sum", "no").toString() == "no" ? false : true;
@@ -465,7 +470,6 @@ void Settings::loadConfig()
     compression = settings.value("compression", "lz4").toString();
     mksq_opt = settings.value("mksq_opt").toString();
     edit_boot_menu = settings.value("edit_boot_menu", "no").toString() == "no" ? false : true;
-    lib_mod_dir = settings.value("lib_mod_dir", "/lib/modules/").toString();
     gui_editor.setFileName(settings.value("gui_editor", "/usr/bin/leafpad").toString());
     stamp = settings.value("stamp").toString();
     force_installer = settings.value("force_installer", "true").toBool();
@@ -488,15 +492,14 @@ void Settings::otherExclusions()
 {
     qDebug() << "+++" << __PRETTY_FUNCTION__ << "+++";
     // add exclusions snapshot and work dirs
-    addRemoveExclusion(true, snapshot_dir.absolutePath());
+    addRemoveExclusion(true, snapshot_dir);
     addRemoveExclusion(true, work_dir);
 
     if (reset_accounts) {
         addRemoveExclusion(true, "/etc/minstall.conf");
         // exclude /etc/localtime if link and timezone not America/New_York
-        if (shell->run("test -L /etc/localtime") && shell->getCmdOut("cat /etc/timezone") != "America/New_York" ) {
+        if (shell->run("test -L /etc/localtime") && shell->getCmdOut("cat /etc/timezone") != "America/New_York" )
             addRemoveExclusion(true, "/etc/localtime");
-        }
     }
 }
 
@@ -505,31 +508,35 @@ void Settings::processArgs(const QCommandLineParser &arg_parser)
     kernel = arg_parser.value("kernel");
     preempt = arg_parser.isSet("preempt");
     if (!arg_parser.value("directory").isEmpty() && QFileInfo::exists(arg_parser.value("directory")))
-        snapshot_dir.setPath(arg_parser.value("directory") + "/snapshot");
+        snapshot_dir = arg_parser.value("directory") + (snapshot_dir.endsWith("/") ? "snapshot" : "/snapshot");
     if (!arg_parser.value("file").isEmpty())
         snapshot_name = arg_parser.value("file") + (arg_parser.value("file").endsWith(".iso") ? QString() : ".iso");
     else
         snapshot_name = getFilename();
     reset_accounts = arg_parser.isSet("reset");
-    if (reset_accounts) excludeAll();
-    if (arg_parser.isSet("month")) reset_accounts = true;
+    if (reset_accounts)
+        excludeAll();
+    if (arg_parser.isSet("month"))
+        reset_accounts = true;
     if (arg_parser.isSet("checksums") || arg_parser.isSet("month"))
         make_chksum = true;
     if (arg_parser.isSet("no-checksums"))
         make_chksum = false;
     if (!arg_parser.value("compression").isEmpty())
         compression = arg_parser.value("compression");
+    if (!arg_parser.value("compression-level").isEmpty())
+        mksq_opt = arg_parser.value("compression-level");
     selectKernel();
 }
 
 void Settings::processExclArgs(const QCommandLineParser &arg_parser)
 {
-    if(!arg_parser.values("exclude").isEmpty()) {
+    if (!arg_parser.values("exclude").isEmpty()) {
         QStringList options = arg_parser.values("exclude");
         QStringList valid_options {"Desktop", "Documents", "Downloads", "Music", "Networks", "Pictures", "Videos"};
-        for (const QString &option : options ) {
-            if (valid_options.contains(option)) excludeItem(option);
-        }
+        for (const QString &option : options )
+            if (valid_options.contains(option))
+                excludeItem(option);
     }
 }
 
@@ -543,7 +550,8 @@ void Settings::setMonthlySnapshot(const QCommandLineParser &arg_parser)
         name = "antiX_" + QString(i686 ? "386" : "x64");
     }
     if (arg_parser.value("file").isEmpty())
-        snapshot_name = name.section("_", 0, 0) + "_" + QDate::currentDate().toString("MMMM") + "_" + name.section("_", 1, 1) + ".iso";
+        snapshot_name = name.section("_", 0, 0) + "_" + QDate::currentDate().toString("MMMM") + "_"
+                + name.section("_", 1, 1) + ".iso";
     if (arg_parser.value("compression").isEmpty())
         compression = "xz";
     reset_accounts = true;
